/** 
 * @file  AquaNetworkSystem.h
 * @brief lbg[NVXeNXwb_
 * @note lbg[Nւ鏈s܂<br>
 * @date 2008/03/14
 */
#ifndef _AQUA_NETWORK_SYSTEM_H_
#define _AQUA_NETWORK_SYSTEM_H_

#ifdef WIN32
#include <winsock2.h>
#include <ws2tcpip.h>
#else // WIN32
#include <sys/socket.h>
#include <unistd.h>
#endif // WIN32

#define USEING_LIBEVENT
#ifdef WIN32
//#define USEING_WINSOCK_SELECT
#endif // WIN32

#ifdef USEING_LIBEVENT
#include "event.h"
#endif // USEING_LIBEVENT

#include <vector>

// ws2tcpip.h defined.
#ifndef NI_MAXHOST
#define NI_MAXHOST  1025  /* Max size of a fully-qualified domain name */
#endif // NI_MAXHOST

#ifndef NI_MAXSERV
#define NI_MAXSERV    32  /* Max size of a service name */
#endif // NI_MAXSERV

#include <AquaTypedef.h>
#include "CryptFunction.h"

#define RECV_BUFFER_SIZE_DEFALUT	20000
#define SEND_BUFFER_SIZE_DEFALUT	20000

namespace AquaNetworkSystem {

	typedef int (*ty_cbfunc_Event)(class CAquaNetworkSession* session, short ev, void* pParam);
	typedef int (*ty_cbfunc_UDPEvent)(class CAquaNetworkUDPSession* session, short ev, void* pParam);

	/// ÍASY
	enum eCryptoAlgorithm
	{
		CRYPTO_ALGORITHM_NONE = 0,
		CRYPTO_ALGORITHM_3DES,
		CRYPTO_ALGORITHM_BLOWFISH,
	};
	#define CRYPTO_ALGORITHM_LIST_MAX	3

	/// Í̏
	enum eCryptoState
	{
		CRYPTO_STATE_PLAIN = 0,
		CRYPTO_STATE_CRYPTOED,
	};

	/// f[^
	enum eCryptoMessageType
	{
		CRYPTO_MES_TYPE_PLAIN = 0,
		CRYPTO_MES_TYPE_HANDSHAKE,
		CRYPTO_MES_TYPE_CHANGE_CIPHER_SPEC,
		CRYPTO_MES_TYPE_CRYPTOED,
	};
	
	enum eSessionEventType
	{
		SESSION_EVENT_TYPE_FD_ACCEPT = 0x0001,
		SESSION_EVENT_TYPE_FD_CONNECT = 0x0002,
		SESSION_EVENT_TYPE_FD_CLOSE = 0x0004,
		SESSION_EVENT_TYPE_FD_READ = 0x0008,
		SESSION_EVENT_TYPE_FD_WRITE = 0x0010,

		SESSION_EVENT_TYPE_HANDSHAKE_FINISH = 0x0100,
	};


	//==============================================
	// struct tagDataHeader
	//==============================================
	struct tagDataHeader
	{
		unsigned char		m_CryptoMessageType;
		unsigned char		m_MajorProtocolVersion;
		unsigned char		m_MinerProtocolVersion;
		unsigned char		m_body_length_high;
		unsigned char		m_body_length_mid;
		unsigned char		m_body_length_low;
	};

	#define AQUA_NETWORK_SENDLEN_PLAIN_MAX	(20000)
	#define AQUA_NETWORK_RECVLEN_PLAIN_MAX	(20000)
	#define AQUA_NETWORK_SENDLEN_CRYPTO_MAX	(AQUA_NETWORK_SENDLEN_PLAIN_MAX*2)
	#define AQUA_NETWORK_RECVLEN_CRYPTO_MAX	(AQUA_NETWORK_RECVLEN_PLAIN_MAX*2)
	#define AQUA_NETWORK_SENDLEN_MAX		(AQUA_NETWORK_SENDLEN_CRYPTO_MAX + sizeof(tagDataHeader))
	#define AQUA_NETWORK_RECVLEN_MAX		(AQUA_NETWORK_RECVLEN_CRYPTO_MAX + sizeof(tagDataHeader))


	//==============================================
	// struct tagChangeCipherSpecData
	//==============================================
	struct tagChangeCipherSpecData
	{
		unsigned char	m_data;
	};

	/// HandShake
	enum eHandShakeMessageType
	{
		HANDSHAKE_MES_TYPE_CL_HELLO = 0,
		HANDSHAKE_MES_TYPE_SV_HELLO,
		HANDSHAKE_MES_TYPE_SV_KEY_EXCHANGE,
		HANDSHAKE_MES_TYPE_SV_HELLO_DONE,
		HANDSHAKE_MES_TYPE_CL_KEY_EXCHANGE,
		HANDSHAKE_MES_TYPE_FINISH,
	};

	//==============================================
	// struct tagHandShakeHeader
	//==============================================
	struct tagHandShakeHeader
	{
		unsigned char			m_HandShakeMessageType;
		unsigned char			m_body_length_high;
		unsigned char			m_body_length_mid;
		unsigned char			m_body_length_low;
	};

	//==============================================
	// struct tagHandShakeClientHello
	//==============================================
	struct tagHandShakeClientHello
	{
		unsigned char			m_CryptoAlgorithmList_length_high;
		unsigned char			m_CryptoAlgorithmList_length_low;
//		unsigned char			m_CryptoAlgorithmList[];
	};

	//==============================================
	// struct tagHandShakeServerHello
	//==============================================
	struct tagHandShakeServerHello
	{
		unsigned char			m_CryptoAlgorithm;
	};

	//==============================================
	// struct tagHandShakeServerKeyExchange
	//==============================================
	struct tagHandShakeServerKeyExchange
	{
		unsigned char	m_RSAModulus_length_hight;
		unsigned char	m_RSAModulus_length_low;
		unsigned char	m_RSAModulus[RSA_MAX_KEY_LENGTH];			// 1..2^16-1
		unsigned char	m_RSAExponent_length_high;
		unsigned char	m_RSAExponent_length_low;
		unsigned char	m_RSAExponent[RSA_MAX_KEY_LENGTH];			// 1..2^16-1

		// J͖̏
//		unsigned char	m_SignatureAlgorithm;			// ASY(anonymous=00, rsa=01, dsa=02)
//		unsigned char	m_md5_hash[16];
//		unsigned char	m_sha_hash[20];

	};

	//==============================================
	// struct tagHandShakeClientKeyExchange
	//==============================================
	struct tagHandShakeClientKeyExchange
	{
		unsigned char	m_CryptoKey_length_hight;
		unsigned char	m_CryptoKey_length_low;
		unsigned char	m_CryptoKey[EVP_MAX_KEY_LENGTH*10];
		unsigned char	m_CryptoIV_length_hight;
		unsigned char	m_CryptoIV_length_low;
		unsigned char	m_CryptoIV[EVP_MAX_IV_LENGTH*10];
	};

	//==============================================
	// struct tagHandShakeServerHelloDone
	//==============================================
	struct tagHandShakeServerHelloDone
	{
		// none
	};
	//==============================================
	// struct tagHandShakeFinish
	//==============================================
	struct tagHandShakeFinish
	{
		// none
	};

	/// \Pbg
	enum eSokcetState
	{
		SOCKET_STATE_ERROR = -1,
		SOCKET_STATE_CONNECTION_CLOSE = 0,
		SOCKET_STATE_NORMAL = 1,
	};

	typedef std::vector<eCryptoMessageType>	CryptoMessages;
	//==============================================
	// class CAquaNetworkSession
	//==============================================
	class CAquaNetworkSession
	{
		friend class CAquaNetworkSessionManager;
	private:
		bool					m_bUsed;
		struct sockaddr_storage	m_ConnectAddr;										/// ڑAhX
		SOCKET					m_ConnectSocket;									/// \Pbg
		eSokcetState			m_SokcetState;										/// \Pbg

		unsigned char			m_recv_socket_buffer[RECV_BUFFER_SIZE_DEFALUT];		/// ::recv֐p̎Mobt@(ÍɂTCYl2{)
		size_t					m_recv_socket_buffer_read_point;					/// ::recv֐p̎MOobt@ǂݍ݈ʒu
		size_t					m_recv_socket_buffer_write_point;					/// ::recv֐p̎MOobt@݈ʒu
		unsigned char			m_recv_buffer[RECV_BUFFER_SIZE_DEFALUT];			/// CAquaNetworkSession::recv֐p̎MOobt@
		size_t					m_recv_buffer_read_point;							/// CAquaNetworkSession::recv֐p̎MOobt@ǂݍ݈ʒu
		size_t					m_recv_buffer_write_point;							/// CAquaNetworkSession::recv֐p̎MOobt@݈ʒu

		unsigned char			m_send_buffer[SEND_BUFFER_SIZE_DEFALUT];			/// send֐p̑MOobt@
		size_t					m_send_buffer_read_point;							/// send֐p̑MOobt@ǂݍ݈ʒu
		size_t					m_send_buffer_write_point;							/// send֐p̑MOobt@݈ʒu

#ifdef USEING_LIBEVENT
		struct event			m_ConnectLibEventRead;								/// MCxgpIuWFNg
		struct event			m_ConnectLibEventWrite;								/// MCxgp
		bool					m_bConnectLibEventReadFlag;							/// MCxgpIuWFNggptO
		bool					m_bConnectLibEventWriteFlag;						/// MCxgpIuWFNggptO
#endif // USEING_LIBEVENT

		ty_cbfunc_Event			m_cbfunc_Event;										/// MR[obN֐ւ̃|C^
		void*					m_cbfunc_EventParam;								/// MR[obN֐̈
		void*					m_ExtensionObject;									/// gIuWFNg
		eCryptoAlgorithm		m_CryptoAlgorithm;									/// ÍASY
		eCryptoState			m_CryptoStateRecv;									/// Í̏(Recv)
		eCryptoState			m_CryptoStateSend;									/// Í̏(Send)

		CryptoMessages			m_LastRecvCryptoMessages;							/// ŏIrecv֐ōsCryptoMessage

		unsigned char			m_CryptoKey[EVP_MAX_KEY_LENGTH];					/// ʐMpÍ
		int						m_CryptoKeyLength;									/// ʐMpÍ̒(0Ŗgp)
		unsigned char			m_CryptoIV[EVP_MAX_IV_LENGTH];						/// ʐMpÍxNg
		int						m_CryptoIVLength;									/// ʐMpÍxNg̒(0Ŗgp)

		unsigned char			m_RSAPrivateKey[RSA_MAX_KEY_LENGTH];				/// ʐMpÍp̔閧ÍRSA閧
		int						m_RSAPrivateKeyLength;								/// ʐMpÍp̔閧ÍRSA閧̒(0Ŗgp)
		unsigned char			m_RSAPublicKey[RSA_MAX_KEY_LENGTH];					/// ʐMpÍp̔閧ÍRSAJ
		int						m_RSAPublicKeyLength;								/// ʐMpÍp̔閧ÍRSAJ̒(0Ŗgp)
		unsigned long			m_RSAPublicExponent;								/// ʐMpÍp̔閧ÍRSAJ̈Íw


		/// MOobt@̏݃TCY擾
		size_t					get_recv_socket_buffer_size(void);
		/// MOobt@̋󂫂擾
		size_t					get_recv_socket_buffer_free(void);
		int						parse_recv_socket_buffer_sub(const tagDataHeader& DataHeader, const unsigned char* Body);

		/// MOobt@̏݃TCY擾
		size_t					get_recv_buffer_size(void);
		/// MOobt@̋󂫂擾
		size_t					get_recv_buffer_free(void);
		/// MOobt@ɒǋL
		size_t					push_recv_buffer(const unsigned char* buffer, size_t size);
		/// MOobt@f[^o
		size_t					pop_recv_buffer(unsigned char* buffer, size_t size);

		/// M֐
		int						recv(void);
		/// 
		int						parse_recv_socket_buffer(void);

		// Mf[^iÍf[^j
		int						Plain(const tagDataHeader& DataHeader, const unsigned char* body);
		// Mf[^inhVF[Nj
		int						HandShake(const tagDataHeader& DataHeader, const unsigned char* body);
		// Mf[^iÍ؂ւj
		int						ChangeCipherSpec(const tagDataHeader& DataHeader, const unsigned char* body);
		// Mf[^iÍf[^j
		int						Cryptoed(const tagDataHeader& DataHeader, const unsigned char* body);

		/// nhVF[NbZ[W(ClientHello)𑗐M
		int						SendHandShakeClientHello(void);
		
		/// nhVF[NbZ[W(ClientHello)̎M
		int						HandShakeClientHello(const tagHandShakeHeader& HandShakeHeader, const unsigned char* HandShakeBody);
		/// nhVF[NbZ[W(ServerHello)̎M
		int						HandShakeServerHello(const tagHandShakeHeader& HandShakeHeader, const unsigned char* HandShakeBody);
		/// nhVF[NbZ[W(ServerKeyExchange)̎M
		int						HandShakeServerKeyExchange(const tagHandShakeHeader& HandShakeHeader, const unsigned char* HandShakeBody);
		/// nhVF[NbZ[W(ServerHelloDone)̎M
		int						HandShakeServerHelloDone(const tagHandShakeHeader& HandShakeHeader, const unsigned char* HandShakeBody);
		/// nhVF[NbZ[W(ClientKeyExchange)̎M
		int						HandShakeClientKeyExchange(const tagHandShakeHeader& HandShakeHeader, const unsigned char* HandShakeBody);
		/// nhVF[NbZ[W(Finish)̎M
		int						HandShakeFinish(const tagHandShakeHeader& HandShakeHeader, const unsigned char* HandShakeBody);

		/// MR[obN֐̌Ăяo
		int						call_cbfunc_Event(short ev);
		/// ŏIrecv֐ōsCryptoMessage擾
		CryptoMessages			GetLastRecvCryptoMessages(void) const;
		/// ÍASY擾
		eCryptoAlgorithm		GetCryptoAlgorithm(void) const;
		/// ÍASYݒ肷
		void					SetCryptoAlgorithm(eCryptoAlgorithm CryptoAlgorithm);
		/// ÍԂݒ肷(Recv)
		void					SetCryptoStateRecv(eCryptoState CryptoState);
		/// ÍԂݒ肷(Send)
		void					SetCryptoStateSend(eCryptoState CryptoState);

	public:
		CAquaNetworkSession();
		virtual ~CAquaNetworkSession();

		/// gptO擾
		bool					IsUsed() const;
		/// gptOݒ
		void					SetUsed(bool bFlag);
		/// \Pbg擾
		SOCKET					GetSocket(void);
		/// \Pbg擾(const)
		SOCKET					GetSocket(void) const;
		/// \PbgAhX擾
		struct sockaddr*		GetSocketAddr(struct sockaddr_storage* pSocketAddr) const;

#ifdef USEING_LIBEVENT
		/// MCxg̐ݒ
		void					SetLibEventRead(short libevent, void(*fn)(int, short, void *), void* arg);
		/// MCxg̒ǉ
		int						AddLibEventRead(struct timeval* tv);
		/// MCxg̍폜
		int						DelLibEventRead(void);
		/// MCxg̐ݒ
		void					SetLibEventWrite(short libevent, void(*fn)(int, short, void *), void* arg);
		/// MCxg̒ǉ
		int						AddLibEventWrite(struct timeval* tv);
		/// MCxg̍폜
		int						DelLibEventWrite(void);
#endif // USEING_LIBEVENT


		/// ZbV̐ڑM
		SOCKET						accept(class CAquaNetworkSessionListen* pAquaNetworkSessionListen);
		/// ZbV̐ڑ
		SOCKET						connect(bool bSecure, const char* hostname, const char* service, ty_cbfunc_Event cbfunc_Event, void* cbfunc_EventParam);
		/// ZbV̐ڑ
		SOCKET						connect(const char* hostname, const char* service, ty_cbfunc_Event cbfunc_Event, void* cbfunc_EventParam);
		/// ZbV̐ڑ
		SOCKET						connect_secure(const char* hostname, const char* service, ty_cbfunc_Event cbfunc_Event, void* cbfunc_EventParam);

		/// ZbV̐ؒf
		void						close(void);
		/// ÍԂ擾(Recv)
		eCryptoState				GetCryptoStateRecv(void) const;
		/// ÍԂ擾(Send)
		eCryptoState				GetCryptoStateSend(void) const;

		/// ʐMpÍ擾
		int							GetCryptoKey(unsigned char* CryptoKey, int CryptoKeyLength) const;
		/// ʐMpÍݒ
		int							SetCryptoKey(const unsigned char* CryptoKey, int CryptoKeyLength);
		/// ʐMpÍxNg擾
		int							GetCryptoIV(unsigned char* CryptoIV, int CryptoIVLength) const;
		/// ʐMpÍxNgݒ
		int							SetCryptoIV(const unsigned char* CryptoIV, int CryptoIVLength);
		/// ʐM閧ÍpRSA閧擾
		int							GetRSAPrivateKey(unsigned char* RSAPrivateKey, int RSAPrivateKeyLength) const;
		/// ʐM閧ÍpRSA閧ݒ
		int							SetRSAPrivateKey(const unsigned char* RSAPrivateKey, int RSAPrivateKeyLength);
		/// ʐM閧ÍpRSAJ擾
		int							GetRSAPublicKey(unsigned char* RSAPublicKey, int RSAPublicKeyLength) const;
		/// ʐM閧ÍpRSAJݒ
		int							SetRSAPublicKey(const unsigned char* RSAPublicKey, int RSAPublicKeyLength);
		/// ʐM閧ÍpRSAJ̈Íw擾
		unsigned long				GetRSAPublicExponent(void) const;
		/// ʐM閧ÍpRSAJ̈Íwݒ
		void						SetRSAPublicExponent(unsigned long RSAPublicExponent);

		/// M֐
		int							recv(unsigned char* buf, int num, int flags);
		/// M֐
		int							send(const unsigned char* buf, int num, int flags);

#ifdef USEING_WINSOCK_SELECT
		/// WinSockSelect
		int							WSAAsyncSelect(HWND hWnd, u_int wMsg, long lEvent);
#endif // USEING_WINSOCK_SELECT
		/// gIuWFNg֘At
		void						attach_ExtensionObject(void* object);
		/// gIuWFNg֘At
		void						detach_ExtensionObject(void);
		/// gIuWFNg擾
		void*						get_ExtensionObject(void);
		/// gIuWFNg擾(const)
		const void*					get_ExtensionObject(void) const;

	};

	//==============================================
	// class CAquaNetworkSessionListen
	//==============================================
	class CAquaNetworkSessionListen
	{
	private:
		bool					m_bUsed;									/// gptO
		SOCKET					m_ListenSocket;								/// listenp\Pbg
#ifdef USEING_LIBEVENT
		struct event			m_ListenLibEvent;							/// listenpҋ@CxgpIuWFNg
		bool					m_bListenLibEventFlag;						/// listenpҋ@CxgpIuWFNggptO
#endif // USEING_LIBEVENT
		char					m_ListenService[NI_MAXSERV];				/// ListenT[rX
		struct sockaddr_storage	m_ListenAddr;								/// listenp\PbgAhX
		ty_cbfunc_Event			m_cbfunc_Event;								/// MR[obN֐ւ̃|C^
		void*					m_cbfunc_EventParam;						/// MR[obN֐̈
		eCryptoAlgorithm		m_CryptoAlgorithm;							/// ʐMÍASY

		unsigned char			m_RSAPrivateKey[RSA_MAX_KEY_LENGTH];		/// ʐM閧ÍpRSA閧
		int						m_RSAPrivateKeyLength;						/// ʐM閧ÍpRSA閧̒(0Ŗgp)
		unsigned char			m_RSAPublicKey[RSA_MAX_KEY_LENGTH];			/// ʐM閧ÍpRSAJ
		int						m_RSAPublicKeyLength;						/// ʐM閧ÍpRSAJ̒(0Ŗgp)
		unsigned long			m_RSAPublicExponent;						/// ʐM閧ÍpRSAJ̈Íw

	public:
		CAquaNetworkSessionListen();
		virtual ~CAquaNetworkSessionListen();

		/// gptO擾
		bool						IsUsed() const;
		/// gptOݒ
		void						SetUsed(bool bFlag);

		/// ListenT[rX擾
		const char*					GetListenService(void) const;
		/// ListenT[rXݒ肷
		void						SetListenService(const char* service);

		/// MR[obN֐|C^擾
		ty_cbfunc_Event				Get_cbfunc_Event(void);
		/// MR[obN֐|C^擾(const)
		const ty_cbfunc_Event		Get_cbfunc_Event(void) const;
		/// MR[obN֐̈|C^擾
		void*						Get_cbfunc_EventParam(void);
		/// MR[obN֐̈|C^擾(const)
		const void*					Get_cbfunc_EventParam(void) const;

		/// listenp\Pbg擾
		SOCKET						GetSocket(void);
		/// listenp\Pbg擾(const)
		SOCKET						GetSocket(void) const;
		/// ʐMÍASYʂ擾
		eCryptoAlgorithm			GetCryptoAlgorithm(void) const;

		/// ʐM閧ÍpRSA閧
		int							GenerateRSAKey(int bits, unsigned long e);
		/// ʐM閧ÍpRSA閧擾
		int							GetRSAPrivateKey(unsigned char* RSAPrivateKey, int RSAPrivateKeyLength) const;
		/// ʐM閧ÍpRSA閧ݒ
		int							SetRSAPrivateKey(const unsigned char* RSAPrivateKey, int RSAPrivateKeyLength);
		/// ʐM閧ÍpRSAJ擾
		int							GetRSAPublicKey(unsigned char* RSAPublicKey, int RSAPublicKeyLength) const;
		/// ʐM閧ÍpRSAJݒ
		int							SetRSAPublicKey(const unsigned char* RSAPublicKey, int RSAPublicKeyLength);
		/// ʐM閧ÍpRSAJ̈Íw擾
		unsigned long				GetRSAPublicExponent(void) const;
		/// ʐM閧ÍpRSAJ̈Íwݒ
		void						SetRSAPublicExponent(unsigned long RSAPublicExponent);

#ifdef USEING_LIBEVENT
		/// listenpҋ@CxgpIuWFNgݒ
		void						SetLibEvent(short libevent, void(*fn)(int, short, void *), void* arg);
		/// listenpҋ@CxgpIuWFNgǉ
		int							AddLibEvent(struct timeval* tv);
		/// listenpҋ@CxgpIuWFNg폜
		int							DelLibEvent(void);
#endif // USEING_LIBEVENT
		/// \Pbgڑ󂯂
		SOCKET						Accept(struct sockaddr *addr, socklen_t *addrlen);
		/// \Pbgڑ̑ҋ@Jn
		SOCKET						Listen(struct addrinfo* ai, int family, int backlog, ty_cbfunc_Event cbfunc_Event, void* cbfunc_EventParam, eCryptoAlgorithm CryptoAlgorithm = CRYPTO_ALGORITHM_NONE);
		/// \Pbgڑ̑ҋ@I
		void						Close(void);

	};

	//==============================================
	// class CAquaNetworkUDPSession
	//==============================================
	class CAquaNetworkUDPSession
	{
		friend class CAquaNetworkUDPSessionManager;
		friend class CAquaNetworkSessionManager;
	private:
		bool					m_bUsed;
		bool					m_bBind;									/// BindtO
		SOCKET					m_Socket;									/// \Pbg
//		struct sockaddr_storage	m_SocketAddr;								/// AhX
		eSokcetState			m_SokcetState;								/// \Pbg
		int						m_family;									/// \PbggpAhXt@~[

#ifdef USEING_LIBEVENT
		struct event			m_LibEventRead;								/// MCxgpIuWFNg
		struct event			m_LibEventWrite;							/// MCxgp
		bool					m_bLibEventReadFlag;						/// MCxgpIuWFNggptO
		bool					m_bLibEventWriteFlag;						/// MCxgpIuWFNggptO
#endif // USEING_LIBEVENT

		char					m_BindService[NI_MAXSERV];					/// BindT[rX
		ty_cbfunc_UDPEvent		m_cbfunc_UDPEvent;							/// MR[obN֐ւ̃|C^
		void*					m_cbfunc_UDPEventParam;						/// MR[obN֐̈
		void*					m_ExtensionObject;							/// gIuWFNg

		/// MR[obN֐̌Ăяo
		int						call_cbfunc_UDPEvent(short ev);

	public:
		CAquaNetworkUDPSession();
		virtual ~CAquaNetworkUDPSession();

		/// gptO擾
		bool					IsUsed() const;
		/// gptOݒ
		void					SetUsed(bool bFlag);

		/// BindSessionǂ
		bool					IsBind() const;
		// BindT[rX擾
		const char*				GetBindService(void) const;
		// BindT[rXݒ
		void					SetBindService(const char* service);

		/// \Pbg擾
		SOCKET					GetSocket(void);
		/// \Pbg擾(const)
		SOCKET					GetSocket(void) const;

#ifdef USEING_LIBEVENT
		/// MCxg̐ݒ
		void					SetLibEventRead(short libevent, void(*fn)(int, short, void *), void* arg);
		/// MCxg̒ǉ
		int						AddLibEventRead(struct timeval* tv);
		/// MCxg̍폜
		int						DelLibEventRead(void);
		/// MCxg̐ݒ
		void					SetLibEventWrite(short libevent, void(*fn)(int, short, void *), void* arg);
		/// MCxg̒ǉ
		int						AddLibEventWrite(struct timeval* tv);
		/// MCxg̍폜
		int						DelLibEventWrite(void);
#endif // USEING_LIBEVENT

		/// UDPZbV쐬
		SOCKET				make_socket(int family, ty_cbfunc_UDPEvent cbfunc_UDPEvent, void* cbfunc_UDPEventParam);
		/// UDPZbV쐬Abind
		SOCKET				make_bind_socket(struct addrinfo* ai, ty_cbfunc_UDPEvent cbfunc_UDPEvent, void* cbfunc_UDPEventParam);

		/// M֐
		int					recvfrom(void* buf, size_t len, int flags, struct sockaddr* from, socklen_t* fromlen);
		/// M֐
		int					sendto(const void* buf, size_t len, int flags, const struct sockaddr* to, socklen_t salen);

		/// UDPZbV
		void				close(void);

	};

	//==============================================
	// class CAquaNetworkSessionManager
	//==============================================
	class CAquaNetworkSessionManager
	{
	private:
#ifdef USEING_WINSOCK_SELECT
		HWND						m_hWndWinSockSelect;
#endif // USEING_WINSOCK_SELECT
#ifdef USEING_LIBEVENT
#ifdef WIN32
		HANDLE						m_hLibEventThread;						/// \PbgCxgpXbhnh
		unsigned int				m_LibEventThreadId;						/// \PbgCxgpXbhID
#else // WIN32
		pthread_t					m_hLibEventThread;						/// \PbgCxgpXbhnh
#endif // WIN32
#endif // USEING_LIBEVENT

#ifdef WIN32
		HANDLE						m_ParseRecvSocketBufferThreadHandle;	/// Mf[^p̓Xbhnh
		unsigned int				m_ParseRecvSocketBufferThreadId;		/// Mf[^p̓XbhID
#else // WIN32
		pthread_t					m_ParseRecvSocketBufferThreadHandle;	/// Mf[^p̓Xbhnh
		pthread_mutex_t				m_ParseRecvSocketBufferThreadMutex;		/// Mf[^p̓XbhTXyhpIuWFNg
		pthread_cond_t				m_ParseRecvSocketBufferThreadCond;		/// Mf[^p̓XbhTXyhpIuWFNg
#endif // WIN32

		int							m_SessionArraySize;						/// ZbVpIuWFNg̍ő吔
		CAquaNetworkSession*		m_SessionArray;							/// ZbVpIuWFNgz
		int							m_SessionListenArraySize;				/// ZbVp҂󂯃IuWFNg̍ő吔
		CAquaNetworkSessionListen*	m_SessionListenArray;					/// ZbVp҂󂯃IuWFNgz
		int							m_UDPSessionArraySize;					/// UDPZbVpIuWFNg̍ő吔
		CAquaNetworkUDPSession*		m_UDPSessionArray;						/// UDPZbVpIuWFNgz

#ifdef USEING_LIBEVENT
#ifdef WIN32
		/// LibEvent̔MpXbh֐
		static DWORD				DispatchLibEventThreadFunc(LPVOID lpvoid);
#else // WIN32
		/// LibEvent̔MpXbh֐
		static void*				DispatchLibEventThreadFunc(LPVOID lpvoid);
#endif // WIN32
#endif // USEING_LIBEVENT

#ifdef USEING_LIBEVENT
#ifdef WIN32
		/// Mf[^͗pXbh֐
		static DWORD				ParseRecvSocketBufferThreadFunc(LPVOID lpvoid);
#else // WIN32
		/// Mf[^͗pXbh֐
		static void*				ParseRecvSocketBufferThreadFunc(LPVOID lpvoid);
#endif // WIN32
#endif // USEING_LIBEVENT
		/// Mf[^
		int							ParseRecvSocketBuffer(void);

		/// \Pbgڑ󂯂
		CAquaNetworkSession*		Accept(CAquaNetworkSessionListen* pAquaNetworkSessionListen);
		/// \PbgIDZbVp҂󂯃IuWFNg擾
		CAquaNetworkSessionListen*	GetAquaNetworkSessionListen(SOCKET s);
		/// \PbgIDZbVpIuWFNg擾
		CAquaNetworkSession*		GetAquaNetworkSession(SOCKET s);
		/// \PbgIDZbVpIuWFNg擾
		CAquaNetworkUDPSession*		GetAquaNetworkUDPSession(SOCKET s);

#ifdef USEING_WINSOCK_SELECT
		/// \PbgCxgEBhER[obN֐
		static LRESULT CALLBACK WinSockProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
#endif // USEING_WINSOCK_SELECT
#ifdef USEING_LIBEVENT
		/// \PbgCxgR[obN֐(accept)
		static void event_accept(const int listenfd, short libevent, void *argv);
		/// \PbgCxgR[obN֐(read)
		static void event_read(const int listenfd, short libevent, void *argv);
		/// \PbgCxgR[obN֐(write)
		static void event_write(const int listenfd, short libevent, void *argv);
		/// \PbgCxgR[obN֐(read)
		static void udp_event_read(const int listenfd, short libevent, void *argv);
		/// \PbgCxgR[obN֐(write)
		static void udp_event_write(const int listenfd, short libevent, void *argv);
#endif // USEING_LIBEVENT

	public:
		CAquaNetworkSessionManager();
		virtual ~CAquaNetworkSessionManager();

		int						CreateSessions(int session_max);
		int						CreateListenSessions(int listen_session_max);
		int						CreateUDPSessions(int udp_session_max);

		void					ReleaseSessions(void);
		void					ReleaseListenSessions(void);
		void					ReleaseUDPSessions(void);

#ifdef USEING_WINSOCK_SELECT
		/// WinSockSelect̏
		int						InitWinSockSelect(HINSTANCE hInstance);
#endif // USEING_WINSOCK_SELECT
#ifdef USEING_LIBEVENT
		/// LibEvent̏
		int						InitLibEvent(void);
		/// LibEvent̔M
		int						DispatchLibEvent(void);
#endif // USEING_LIBEVENT

		/// Mf[^͗pXbhJn
		int						CreateParseRecvSocketBufferThread(void);
		/// Mf[^͗pXbhTXyhԂɂ
		void					SuspendParseRecvSocketBufferThread(void);
		/// Mf[^͗pXbhTXyhԂ
		void					ResumeParseRecvSocketBufferThread(void);

		/// ZbV̑҂󂯂Jn
		int						Listen(const char* service, int family, int backlog, ty_cbfunc_Event cbfunc_Event, void* cbfunc_EventParam, eCryptoAlgorithm CryptoAlgorithm = CRYPTO_ALGORITHM_NONE);
		void					CloseListen(const char* service);

		/// ZbVƂ̐ڑJn
		CAquaNetworkSession*	Connect(bool bSecure, const char* hostname, const char* service, ty_cbfunc_Event cbfunc_Event, void* cbfunc_EventParam);
		/// ZbVƂ̐ڑJn
		CAquaNetworkSession*	Connect(const char* hostname, const char* service, ty_cbfunc_Event cbfunc_Event, void* cbfunc_EventParam);
		/// ZbVƂ̐ڑJn
		CAquaNetworkSession*	ConnectSecure(const char* hostname, const char* service, ty_cbfunc_Event cbfunc_Event, void* cbfunc_EventParam);

		/// ZbVƂ̐ڑؒf
		int						Close(CAquaNetworkSession* session);

		/// UDPZbVJ
		CAquaNetworkUDPSession*	OpenUDPSession(int family, ty_cbfunc_UDPEvent cbfunc_UDPEvent, void* cbfunc_UDPEventParam);
		/// UDPZbV
		void					CloseUDPSession(CAquaNetworkUDPSession* session);
		/// UDPBindZbVJ
		int						OpenUDPBindSession(const char* service, int family, ty_cbfunc_UDPEvent cbfunc_UDPEvent, void* cbfunc_UDPEventParam);
		/// UDPBindZbV
		void					CloseUDPBindSession(const char* service);

	};

}

#endif // _AQUA_NETWORK_SYSTEM_H_
