#include <stdio.h>
#include <sys/socket.h>
#include <netdb.h>
#include "AquaMessageSystem.h"

#define RECV_BUUFER_SIZE				4096

static void addr2str(const struct sockaddr *sa, socklen_t salen, char* host_str, size_t host_strlen)
{
	if(NULL == sa) { return; }

	char host[NI_MAXHOST] = "";
	char serv[NI_MAXSERV] = "";
	int error = 0;

	if(AF_INET6 == sa->sa_family) {
		// ipv6
		error = getnameinfo(sa, sizeof(struct sockaddr_in6), host, sizeof(host), serv, sizeof(serv),
							NI_NUMERICHOST | NI_NUMERICSERV);
		if(0 == error) {
			snprintf(host_str, host_strlen, "%s:%s(ipv6)", host, serv);
		} else {
			printf("getnameinfo : %s\n", gai_strerror(error));
			snprintf(host_str, host_strlen, "unknown");
		}
	} else if(AF_INET == sa->sa_family) {
		// ipv4
		error = getnameinfo(sa, sizeof(struct sockaddr_in), host, sizeof(host), serv, sizeof(serv),
							NI_NUMERICHOST | NI_NUMERICSERV);
		if(0 == error) {
			snprintf(host_str, host_strlen, "%s:%s(ipv4)", host, serv);
		} else {
			printf("getnameinfo : %s\n", gai_strerror(error));
			snprintf(host_str, host_strlen, "unknown");
		}
	} else if(AF_UNSPEC == sa->sa_family) {
		snprintf(host_str, host_strlen, "unknown");
	} else {
		snprintf(host_str, host_strlen, "unknown");
	}
}
static void addr2str(const struct sockaddr *sa, socklen_t salen, char* host, size_t hostlen, char* serv, size_t servlen)
{
	if(NULL == sa) { return; }
	if(NULL == host) { return; }
	if(NULL == serv) { return; }

	int error = 0;

	if(AF_INET6 == sa->sa_family) {
		// ipv6
		error = getnameinfo(sa, sizeof(struct sockaddr_in6), host, hostlen, serv, servlen,
							NI_NUMERICHOST | NI_NUMERICSERV);
		if(0 != error) {
			printf("getnameinfo : %s\n", gai_strerror(error));
			snprintf(host, hostlen, "unknown");
			snprintf(serv, servlen, "unknown");
		}
	} else if(AF_INET == sa->sa_family) {
		// ipv4
		error = getnameinfo(sa, sizeof(struct sockaddr_in), host, hostlen, serv, servlen,
							NI_NUMERICHOST | NI_NUMERICSERV);
		if(0 != error) {
			printf("getnameinfo : %s\n", gai_strerror(error));
			snprintf(host, hostlen, "unknown");
			snprintf(serv, servlen, "unknown");
		}
	} else if(AF_UNSPEC == sa->sa_family) {
		snprintf(host, hostlen, "unknown");
		snprintf(serv, servlen, "unknown");
	} else {
		snprintf(host, hostlen, "unknown");
		snprintf(serv, servlen, "unknown");
	}
}

int recv_user_message(CAquaMessageSession* session, const char* hostname, const char* service, const void* pMessageData, size_t nMessageData)
{
	TCHAR szBuff[RECV_BUUFER_SIZE+256] = "\0";

	TCHAR timestamp[32] = "\0";
	time_t now_time;							// time֐p
	struct tm *now_time_local;
	time(&now_time);							// 擾
	now_time_local = localtime(&now_time);		// nԂ̍\̂ɕϊ

	memset(timestamp, '\0', sizeof(timestamp));
	snprintf(timestamp, sizeof(timestamp)-1, "%04d-%02d-%02d %02d:%02d:%02d",
	 now_time_local->tm_year + 1900,
	 now_time_local->tm_mon + 1,
	 now_time_local->tm_mday,
	 now_time_local->tm_hour,
	 now_time_local->tm_min,
	 now_time_local->tm_sec);

	// recv from user message
	tagMessageDataUserMessage MessageDataUserMessage;
	memcpy(&MessageDataUserMessage, pMessageData, sizeof(tagMessageDataUserMessage));


	FILE* pFile = fopen("ServerMessage.log", "at");
	if(NULL != pFile) {
		memset(szBuff, '\0', sizeof(szBuff));
		snprintf(szBuff, sizeof(szBuff) - 1, " [%s]  USER_MESSAGE data %s", timestamp, MessageDataUserMessage.m_message);
		fprintf(pFile, "%s\n", szBuff);

		fclose(pFile);
	}

	// message echo back to client
	if(session->is_session_used()) {
		session->send_message(&MessageDataUserMessage, sizeof(tagMessageDataUserMessage));
	}
	if(session->is_udp_session_used()) {
		session->sendto_udp_message(&MessageDataUserMessage, sizeof(tagMessageDataUserMessage), hostname, service);
	}

	return 1;
}
int session_event(CAquaMessageSession* session, const char* hostname, const char* service, const void* pMessageData, size_t nMessageData)
{
	TCHAR szBuff[RECV_BUUFER_SIZE+256] = "\0";

	TCHAR timestamp[32] = "\0";
	time_t now_time;							// time֐p
	struct tm *now_time_local;
	time(&now_time);							// 擾
	now_time_local = localtime(&now_time);		// nԂ̍\̂ɕϊ

	memset(timestamp, '\0', sizeof(timestamp));
	snprintf(timestamp, sizeof(timestamp)-1, "%04d-%02d-%02d %02d:%02d:%02d",
	 now_time_local->tm_year + 1900,
	 now_time_local->tm_mon + 1,
	 now_time_local->tm_mday,
	 now_time_local->tm_hour,
	 now_time_local->tm_min,
	 now_time_local->tm_sec);

	// recv from user message
	tagMessageDataSesssionEvent MessageDataSesssionEvent;
	memcpy(&MessageDataSesssionEvent, pMessageData, sizeof(tagMessageDataSesssionEvent));

	FILE* pFile = fopen("ServerAccess.log", "at");
	if(NULL == pFile) {
		return 1;
	}

	struct sockaddr* sa = (struct sockaddr*)&(MessageDataSesssionEvent.m_SessionAddr);

	char m_local_host[NI_MAXHOST];						/// ڑzXg
	char m_local_serv[NI_MAXSERV];						/// ڑT[rX
	char m_foreign_host[NI_MAXHOST];						/// ڑzXg
	char m_foreign_serv[NI_MAXSERV];						/// ڑT[rX

	memset(m_local_host, '\0', sizeof(m_local_host));
	memset(m_local_serv, '\0', sizeof(m_local_serv));
	memset(m_foreign_host, '\0', sizeof(m_foreign_host));
	memset(m_foreign_serv, '\0', sizeof(m_foreign_serv));

	char LocalAddress[NI_MAXHOST+NI_MAXSERV+10] = "unknown";
	char ForeignAddress[NI_MAXHOST+NI_MAXSERV+10] = "unknown";

	if(AF_INET6 == sa->sa_family) {
		// ipv6
		addr2str((struct sockaddr*)sa, sizeof(struct sockaddr_in6), m_foreign_host, sizeof(m_foreign_host), m_foreign_serv, sizeof(m_foreign_serv));
		snprintf(ForeignAddress, sizeof(ForeignAddress), "%s:%s", m_foreign_host, m_foreign_serv);
	} else if(AF_INET == sa->sa_family) {
		// ipv4
		addr2str((struct sockaddr*)sa, sizeof(struct sockaddr_in), m_foreign_host, sizeof(m_foreign_host), m_foreign_serv, sizeof(m_foreign_serv));
		snprintf(ForeignAddress, sizeof(ForeignAddress), "%s:%s", m_foreign_host, m_foreign_serv);
	} else {
		fclose(pFile);
		return 1;
	}

	switch(MessageDataSesssionEvent.m_ev) {
	case AQUAMESSYS_SESSION_EVENT_TYPE_FD_ACCEPT:
		memset(szBuff, '\0', sizeof(szBuff));
		snprintf(szBuff, sizeof(szBuff) - 1, "%s - -  [%s]  SESSION_ACCEPT", ForeignAddress, timestamp);
		fprintf(pFile, "%s\n", szBuff);
		break;
	case AQUAMESSYS_SESSION_EVENT_TYPE_FD_CONNECT:
		memset(szBuff, '\0', sizeof(szBuff));
		snprintf(szBuff, sizeof(szBuff) - 1, "%s - -  [%s]  SESSION_CONNECT", ForeignAddress, timestamp);
		fprintf(pFile, "%s\n", szBuff);
		break;
	case AQUAMESSYS_SESSION_EVENT_TYPE_FD_CLOSE:
		memset(szBuff, '\0', sizeof(szBuff));
		snprintf(szBuff, sizeof(szBuff) - 1, "%s - -  [%s]  SESSION_CLOSE", ForeignAddress, timestamp);
		fprintf(pFile, "%s\n", szBuff);
		break;
	case AQUAMESSYS_SESSION_EVENT_TYPE_FD_READ:
		memset(szBuff, '\0', sizeof(szBuff));
		snprintf(szBuff, sizeof(szBuff) - 1, "%s - -  [%s]  SESSION_READ", ForeignAddress, timestamp);
		fprintf(pFile, "%s\n", szBuff);
		break;
	case AQUAMESSYS_SESSION_EVENT_TYPE_HANDSHAKE_FINISH:
		memset(szBuff, '\0', sizeof(szBuff));
		snprintf(szBuff, sizeof(szBuff) - 1, "%s - -  [%s]  SESSION_HANDSHAKE_FINISH", ForeignAddress, timestamp);
		fprintf(pFile, "%s\n", szBuff);
		break;
	}
	fclose(pFile);

	return 1;
}

int main(void)
{

	CAquaMessageSystem* pAquaMessageSystem = new CAquaMessageSystem();
	if(NULL == pAquaMessageSystem) {
		delete pAquaMessageSystem;
		return 0;
	}
	pAquaMessageSystem->CreateListenSessions(5);
	pAquaMessageSystem->CreateSessions(10);
	pAquaMessageSystem->CreateUDPSessions(10);

	if(0 == pAquaMessageSystem->InitSocketEvent()) {
		delete pAquaMessageSystem;
		return 0;
	}

	tagExtensionCallFunction user_message;

	user_message.m_call_func = recv_user_message;
	user_message.m_type = AQUAMESSYS_MESSAGE_TYPE_USER_EXTENSION_START+1;
	user_message.m_size = sizeof(tagMessageDataUserMessage);
	pAquaMessageSystem->add_ExtensionCallFunction(user_message);

	user_message.m_call_func = session_event;
	user_message.m_type = AQUAMESSYS_MESSAGE_TYPE_SESSION_EVENT;
	user_message.m_size = sizeof(tagMessageDataSesssionEvent);
	pAquaMessageSystem->add_ExtensionCallFunction(user_message);

	int iRet = 0;

//	iRet = pAquaMessageSystem->listen("20000", AF_INET, AQUAMESSYS_CRYPTO_ALGORITHM_NONE);
//	iRet = pAquaMessageSystem->listen("20000", AF_INET6, AQUAMESSYS_CRYPTO_ALGORITHM_NONE);
//	iRet = pAquaMessageSystem->listen("20000", AF_INET, AQUAMESSYS_CRYPTO_ALGORITHM_BLOWFISH);
	iRet = pAquaMessageSystem->listen("20000", AF_INET6, AQUAMESSYS_CRYPTO_ALGORITHM_BLOWFISH);
//	iRet = pAquaMessageSystem->listen("20000", AF_INET, AQUAMESSYS_CRYPTO_ALGORITHM_3DES);
//	iRet = pAquaMessageSystem->listen("20000", AF_INET6, AQUAMESSYS_CRYPTO_ALGORITHM_3DES);
	if(0 == iRet) {
		printf("##error listen()\n");
		delete pAquaMessageSystem;
		return 0;
	}
//	iRet = pAquaMessageSystem->open_udp_bind("20001", PF_INET);
	iRet = pAquaMessageSystem->open_udp_bind("20001", PF_INET6);
	if(0 == iRet) {
		printf("##error open_udp_bind()\n");
		delete pAquaMessageSystem;
		return 0;
	}


	if(0 == pAquaMessageSystem->DispatchSocketEvent()) {
		delete pAquaMessageSystem;
		return 0;
	}

	printf("input \'exit\' or \'quit\' work to exit.\n");
	char szInput[16];

	while(1) {
		memset(szInput, '\0', sizeof(szInput));
		fgets(szInput, sizeof(szInput), stdin);
		szInput[strlen(szInput)-1] = '\0';

		if(0 == strcmp(szInput, "e")) { break; }
		if(0 == strcmp(szInput, "exit")) { break; }
		if(0 == strcmp(szInput, "q")) { break; }
		if(0 == strcmp(szInput, "quit")) { break; }
//		sleep(1);
	}
	delete pAquaMessageSystem;

	return 1;
}


