/** 
 * @file  AquaMessageSystem.h
 * @brief bZ[WVXeNXwb_
 * @note bZ[Wf[^Ɋւ鏈s܂<br>
 * @date 2008/03/14
 */
#ifndef _AQUA_MESSAGE_SYSTEM_H_
#define _AQUA_MESSAGE_SYSTEM_H_

#ifdef WIN32
#ifndef _AQUA_MESSAGE_SYSTEM_MAKE
	#pragma comment(lib, "ws2_32.lib")
	#pragma comment(lib, "User32.lib")
	#ifdef _DLL
		#ifdef _DEBUG
			#pragma comment(lib, "libeay32MDd.lib")
			#pragma comment(lib, "AquaMessageSystemMDd.lib")
		#else // _DEBUG
			#pragma comment(lib, "libeay32MD.lib")
			#pragma comment(lib, "AquaMessageSystemMD.lib")
		#endif // _DEBUG
	#else // _DLL
		#ifdef _DEBUG
			#pragma comment(lib, "libeay32MTd.lib")
			#pragma comment(lib, "AquaMessageSystemMTd.lib")
		#else // _DEBUG
			#pragma comment(lib, "libeay32MT.lib")
			#pragma comment(lib, "AquaMessageSystemMT.lib")
		#endif // _DEBUG
	#endif // _DLL
	#pragma comment(lib, "libevent.lib")
#endif // _AQUA_MESSAGE_SYSTEM_MAKE
#endif // WIN32

#include <stdio.h>
#include <vector>

#ifdef WIN32
#include <winsock2.h>
#include <ws2tcpip.h>
#include <windows.h>
#else // WIN32
#include <sys/socket.h>
#endif // WIN32

#ifdef WIN32
#define CLOSE_SOCKET		::closesocket
#else // WIN32
#endif // WIN32

#include "AquaTypedef.h"

#define		UID_LEN_MAX					256
#define		PASSWD_LEN_MAX				256
#define		USER_MESSAGE_LEN_MAX		1024
#define		USER_STATE_LEN_MAX			64


#ifndef _AQUA_NETWORK_SYSTEM_H_
/**
 * @namespace AquaMessageSystem
 */
namespace AquaNetworkSystem {
	class CAquaNetworkSession;
	class CAquaNetworkUDPSession;
	class CAquaNetworkSessionManager;
}
#endif // _AQUA_NETWORK_SYSTEM_H_

/**
 * @enum eAquaMessageSystemServiceType
 * @brief T[rX
 * @note <br>
 */
enum eAquaMessageSystemServiceType
{
	AQUAMESSYS_SERVICE_TYPE_NONE = 0,	/// `
	AQUAMESSYS_SERVICE_TYPE_SERVER,		/// T[o
	AQUAMESSYS_SERVICE_TYPE_CLIENT,		/// NCAg
};
/**
 * @enum eAquaMessageSystemCryptoAlgorithm
 * @brief ÍASY
 * @note <br>
 */
enum eAquaMessageSystemCryptoAlgorithm
{
	AQUAMESSYS_CRYPTO_ALGORITHM_NONE = 0,	/// Í
	AQUAMESSYS_CRYPTO_ALGORITHM_3DES,		/// 3desÍ
	AQUAMESSYS_CRYPTO_ALGORITHM_BLOWFISH,	/// blowfishÍ
};

/**
 * @enum eAquaMessageSystemMessageType
 * @brief bZ[W
 * @note <br>
 */
enum eAquaMessageSystemMessageType
{
	AQUAMESSYS_MESSAGE_TYPE_UNKNOWN					= 0x10000001,			/// m
	AQUAMESSYS_MESSAGE_TYPE_SESSION_EVENT			= 0x10000002,			/// ZbVCxg
	AQUAMESSYS_MESSAGE_TYPE_HANDSHAKE_FINISH		= 0x10000003,			/// nhVF[NIʒm
	AQUAMESSYS_MESSAGE_TYPE_USER_AUTH_REQUSET		= 0x10000004,			/// Fؗv
	AQUAMESSYS_MESSAGE_TYPE_USER_AUTH_RESULT		= 0x10000005,			/// F،
	AQUAMESSYS_MESSAGE_TYPE_USER_MESSAGE			= 0x10000006,			/// [UbZ[W
	AQUAMESSYS_MESSAGE_TYPE_USER_STATE				= 0x10000007,			/// [U
	AQUAMESSYS_MESSAGE_TYPE_USER_EXTENSION_START	= 0x00000000,			/// gbZ[WpJnʒu
	AQUAMESSYS_MESSAGE_TYPE_USER_EXTENSION_END		= 0x0000ffff,			/// gbZ[WpJnʒu
	AQUAMESSYS_MESSAGE_TYPE_USER_EXTENSION_ALL		= 0x00010000,			/// SgbZ[W
};


/**
 * @enum eAquaMessageSystemSessionEventType
 * @brief ZbVCxg
 * @note <br>
 */
enum eAquaMessageSystemSessionEventType
{
	AQUAMESSYS_SESSION_EVENT_TYPE_FD_ACCEPT = 0x0001,			/// ڑJn̒ʒm
	AQUAMESSYS_SESSION_EVENT_TYPE_FD_CONNECT = 0x0002,			/// ڑ̒ʒm
	AQUAMESSYS_SESSION_EVENT_TYPE_FD_CLOSE = 0x0004,			/// ڑؒf̒ʒm
	AQUAMESSYS_SESSION_EVENT_TYPE_FD_READ = 0x0008,				/// f[^M̒ʒm
	AQUAMESSYS_SESSION_EVENT_TYPE_FD_WRITE = 0x0010,			/// f[^M\̒ʒm

	AQUAMESSYS_SESSION_EVENT_TYPE_HANDSHAKE_FINISH = 0x0100,	/// nhVF[N̒ʒm
};


/**
 * @namespace AquaMessageSystem
 */
namespace AquaMessageSystem {
	//==============================================
	// class CAquaMessageUserAbstract
	//==============================================
	/**
	 * @class CAquaMessageUserAbstract
	 * @brief [UNX(ۃNX)
	 * @note GNX|[gpۃNX<br>
	 */
	class CAquaMessageUserAbstract
	{
	public:
		virtual	LPCTSTR get_uid(void) const = 0;
		virtual size_t	get_uid_size(void) const = 0;
		virtual void	set_uid(LPCTSTR uid, size_t size) = 0;
		virtual LPCTSTR	get_passwd(void) const = 0;
		virtual size_t	get_passwd_size(void) const = 0;
		virtual void	set_passwd(LPCTSTR passwd, size_t size) = 0;
	};
	//==============================================
	// class CAquaMessageUser
	//==============================================
	/**
	 * @class CAquaMessageUser
	 * @brief [UNX
	 * @note [UɊւf[^NX<br>
	 */
	class CAquaMessageUser : public CAquaMessageUserAbstract
	{
	private:
		TCHAR	m_uid[UID_LEN_MAX];				/// uid([UID)
		TCHAR	m_passwd[PASSWD_LEN_MAX];		/// passwd(pX[h)

	public:
		/// RXgN^
		CAquaMessageUser();
		/// fXgN^
		virtual ~CAquaMessageUser();
		/// Rs[RXgN^
		CAquaMessageUser(const CAquaMessageUser& rAquaMessageUser);
		/// Zq̃I[o[Ch
		CAquaMessageUser& operator=(const CAquaMessageUser& rAquaMessageUser);

		/// uid擾
		virtual LPCTSTR	get_uid(void) const;
		/// uid̒擾
		virtual size_t	get_uid_size(void) const;
		/// uidݒ肷
		virtual void	set_uid(LPCTSTR uid, size_t size);
		/// passwd擾
		virtual LPCTSTR	get_passwd(void) const;
		/// passwd̒擾
		virtual size_t	get_passwd_size(void) const;
		/// passwdݒ肷
		virtual void	set_passwd(LPCTSTR passwd, size_t size);

	};

	//==============================================
	// class CAquaMessageUserManagerAbstract
	//==============================================
	/**
	 * @class CAquaMessageUserManagerAbstract
	 * @brief [UǗNX(ۃNX)
	 * @note GNX|[gpۃNX<br>
	 */
	class CAquaMessageUserManagerAbstract
	{
	public:
		virtual int						add_user(LPCTSTR uid, size_t uid_size, LPCTSTR passwd, size_t passwd_size) = 0;
		virtual int						delete_user(LPCTSTR uid, size_t uid_size) = 0;
		virtual int						delete_alluser(void) = 0;
		virtual CAquaMessageUser		reference_user(LPCTSTR uid, size_t uid_size) const = 0;
		virtual CAquaMessageUser*		get_user(LPCTSTR uid, size_t uid_size) = 0;
		virtual const CAquaMessageUser*	get_user(LPCTSTR uid, size_t uid_size) const = 0;

	};
	//==============================================
	// class CAquaMessageUserManager
	//==============================================
	/**
	 * @class CAquaMessageUserManager
	 * @brief [UǗNX
	 * @note [UɊւǗs܂<br>
	 */
	class CAquaMessageUserManager : public CAquaMessageUserManagerAbstract
	{
	private:
		std::vector<CAquaMessageUser>	m_AquaMessageUserArray;			/// [U̔z
		
	public:
		/// RXgN^
		CAquaMessageUserManager();
		/// fXgN^
		virtual ~CAquaMessageUserManager();
		
		/// [U̒ǉ
		int						add_user(LPCTSTR uid, size_t uid_size, LPCTSTR passwd, size_t passwd_size);
		/// wuid([UID)̃[U폜
		int						delete_user(LPCTSTR uid, size_t uid_size);
		/// S[U̒ǉ
		int						delete_alluser(void);
		/// [UQƂ
		CAquaMessageUser		reference_user(LPCTSTR uid, size_t uid_size) const;
		/// [Ȕ擾
		CAquaMessageUser*		get_user(LPCTSTR uid, size_t uid_size);
		/// [Ȕ擾(const)
		const CAquaMessageUser*	get_user(LPCTSTR uid, size_t uid_size) const;

	};

	//==============================================
	// class CAquaMessageSessionAbstract
	//==============================================
	/**
	 * @class CAquaMessageSessionAbstract
	 * @brief bZ[WʐMZbVIuWFNgNX(ۃNX)
	 * @note GNX|[gpۃNX<br>
	 */
	class CAquaMessageSessionAbstract
	{
	private:
//		virtual void	attach_session(AquaNetworkSystem::CAquaNetworkSession* session) = 0;
//		virtual void	detach_session(void) = 0;
//		virtual void	close_session(void) = 0;
//		virtual void	attach_udp_session(AquaNetworkSystem::CAquaNetworkUDPSession* session) = 0;
//		virtual void	detach_udp_session(void) = 0;
//		virtual void	close_udp_session(void) = 0;

	public:
		virtual bool	is_session_used(void) const = 0;
		virtual int		send_message(const void* pMessageData, size_t nMessageDataSize) = 0;
		virtual bool	is_udp_session_used(void) const = 0;
		virtual int		sendto_udp_message(const void* pMessageData, size_t nMessageDataSize, const char* hostname, const char* service) = 0;

	};
	//==============================================
	// class CAquaMessageSession
	//==============================================
	/**
	 * @class CAquaMessageSession
	 * @brief bZ[WʐMZbVIuWFNgNX
	 * @note bZ[WʐMZbVIuWFNg<br>
	 * [UCAquaMessageSystemŊǗꂽZbVAquaNetworkSystem::CAquaNetworkSession͎gp
	 * ԐړIȃZbVNXł邱CAquaMessageSessionNXIuWFNggp
	 */
	class CAquaMessageSession : public CAquaMessageSessionAbstract
	{
	friend class CAquaMessageSystem;
	private:
		AquaNetworkSystem::CAquaNetworkSession*			m_ns;		/// lbg[NZbVIuWFNg|C^
		AquaNetworkSystem::CAquaNetworkUDPSession*		m_ns_udp;	/// lbg[NZbVIuWFNg|C^(UDP)

		/// ZbV֘At
		void	attach_session(AquaNetworkSystem::CAquaNetworkSession* session);
		/// ZbV֘At
		void	detach_session(void);
		/// ֘AtꂽZbV
		void	close_session(void);

		/// UDPZbV֘At
		void	attach_udp_session(AquaNetworkSystem::CAquaNetworkUDPSession* session);
		/// UDPZbV֘At
		void	detach_udp_session(void);
		/// ֘AtꂽUDPZbV
		void	close_udp_session(void);

	public:
		/// RXgN^
		CAquaMessageSession();
		/// fXgN^
		virtual ~CAquaMessageSession();
		/// Rs[RXgN^
		CAquaMessageSession(const CAquaMessageSession& rAquaMessageSession);
		/// ZqI[oCh
		CAquaMessageSession operator=(const CAquaMessageSession& rAquaMessageSession);
		/// ZqI[oCh
		bool operator==(const CAquaMessageSession& rAquaMessageSession) const;
		/// sZqI[oCh
		bool operator!=(const CAquaMessageSession& rAquaMessageSession) const;

		/// ֘AtꂽZbV͗LǂH
		bool	is_session_used(void) const;
		/// ֘AtꂽZbVgpăbZ[W𑗐M
		int		send_message(const void* pMessageData, size_t nMessageDataSize);


		/// ֘AtꂽUDPZbV͗LǂH
		bool	is_udp_session_used(void) const;
		/// ֘AtꂽUDPZbVgpăbZ[W𑗐M
		int		sendto_udp_message(const void* pMessageData, size_t nMessageDataSize, const char* hostname, const char* service);

	};

	/**
	 * @struct tagMessageHeader
	 * @brief bZ[Wf[^wb_
	 * @note <br>
	 */
	struct tagMessageHeader
	{
		unsigned long		m_type;		/// bZ[W
		unsigned short		m_size;		/// bZ[Wf[^̃TCY
	};

	/**
	 * @struct tagMessageDataSesssionEvent
	 * @brief bZ[Wf[^(ZbV)
	 * @note <br>
	 */
	struct tagMessageDataSesssionEvent
	{
		tagMessageHeader		m_header;				/// bZ[Wf[^wb_
		short					m_ev;
		struct sockaddr_storage	m_SessionAddr;			/// ڑAhX
	};

	/**
	 * @struct tagMessageDataUserAuthRequest
	 * @brief bZ[Wf[^(Fؗv)
	 * @note <br>
	 */
	struct tagMessageDataUserAuthRequest
	{
		tagMessageHeader	m_header;					/// bZ[Wf[^wb_
		TCHAR				m_uid[UID_LEN_MAX];			/// uid([UID)
		TCHAR				m_passwd[PASSWD_LEN_MAX];	/// pass(pX[h)
	};

	/**
	 * @struct tagMessageDataUserAuthResult
	 * @brief bZ[Wf[^(F،)
	 * @note <br>
	 */
	struct tagMessageDataUserAuthResult
	{
		tagMessageHeader	m_header;					/// bZ[Wf[^wb_
		int					m_result;					/// F،(1: 0:s)
		TCHAR				m_uid[UID_LEN_MAX];			/// uid([UID)
		TCHAR				m_passwd[PASSWD_LEN_MAX];	/// pass(pX[h)
	};

	/**
	 * @struct tagMessageDataUserMessage
	 * @brief bZ[Wf[^([UbZ[W)
	 * @note <br>
	 */
	struct tagMessageDataUserMessage
	{
		tagMessageHeader	m_header;							/// bZ[Wf[^wb_
		TCHAR				m_uid_from[UID_LEN_MAX];			/// M҂uid([UID)
		TCHAR				m_uid_to[UID_LEN_MAX];				/// M҂uid([UID)
		TCHAR				m_message[USER_MESSAGE_LEN_MAX];	/// M郁bZ[W
	};

	/**
	 * @struct tagMessageDataUserState
	 * @brief bZ[Wf[^([U)
	 * @note <br>
	 */
	struct tagMessageDataUserState
	{
		tagMessageHeader	m_header;							/// bZ[Wf[^wb_
		TCHAR				m_uid_from[UID_LEN_MAX];			/// M҂uid([UID)
		TCHAR				m_uid_to[UID_LEN_MAX];				/// M҂uid([UID)
		TCHAR				m_state[USER_STATE_LEN_MAX];		/// M郆[U
	};

	/**
	 * @struct tagExtensionCallFunction
	 * @brief gbZ[Wf[^`\
	 * @note <br>
	 */
	struct tagExtensionCallFunction
	{
		unsigned long		m_type;		/// bZ[W
		unsigned short		m_size;		/// gbZ[Wf[^̃TCY

		/// gbZ[Wf[^M̃R[obN֐ւ̃|C^
		int					(*m_call_func)(class CAquaMessageSession* session, const char* hostname, const char* service, const void*, size_t);
	};

	//==============================================
	// class CAquaMessageSystemAbstract
	//==============================================
	/**
	 * @class CAquaMessageSystemAbstract
	 * @brief bZ[WVXeNX(ۃNX)
	 * @note GNX|[gpۃNX<br>
	 */
	class CAquaMessageSystemAbstract
	{
	private:
		virtual int						auth(CAquaMessageSession* session, LPCTSTR uid, size_t uid_size, LPCTSTR passwd, size_t passwd_size) = 0;
		virtual int						notify_session_event(AquaNetworkSystem::CAquaNetworkSession* session, short ev) = 0;
		virtual	int						notify_udp_session_event(AquaNetworkSystem::CAquaNetworkUDPSession* session, short ev, const struct sockaddr* sa, socklen_t salen) = 0;

		virtual int						parse_message(AquaNetworkSystem::CAquaNetworkSession* session, const unsigned char* buf, int num) = 0;
		virtual int						parse_message_data_user_auth_request(AquaNetworkSystem::CAquaNetworkSession* session, const void* pMessageData, size_t nMessageData) = 0;
		virtual int						parse_message_data_user_auth_result(AquaNetworkSystem::CAquaNetworkSession* session, const void* pMessageData, size_t nMessageData) = 0;
		virtual int						parse_message_data_user_message(AquaNetworkSystem::CAquaNetworkSession* session, const void* pMessageData, size_t nMessageData) = 0;
		virtual int						parse_message_data_user_state(AquaNetworkSystem::CAquaNetworkSession* session, const void* pMessageData, size_t nMessageData) = 0;
		virtual int						parse_udp_message(AquaNetworkSystem::CAquaNetworkUDPSession* session, const unsigned char* buf, int num, const struct sockaddr* sa, socklen_t salen) = 0;

	public:
		virtual int						CreateSessions(int session_max) = 0;
		virtual int						CreateListenSessions(int listen_session_max) = 0;
		virtual int						CreateUDPSessions(int udp_session_max) = 0;
		virtual void					ReleaseSessions(void) = 0;
		virtual void					ReleaseListenSessions(void) = 0;
		virtual void					ReleaseUDPSessions(void) = 0;
		virtual int						InitSocketEvent(void) = 0;
		virtual int						DispatchSocketEvent(void) = 0;
		virtual int						listen(const char* service, int family, eAquaMessageSystemCryptoAlgorithm CryptoAlgorithm) = 0;
		virtual int						load_auth_user_list(LPCTSTR lpFileName) = 0;
		virtual CAquaMessageSession*	connect(const char* hostname, const char* service) = 0;
		virtual CAquaMessageSession*	connect(const char* hostname, const char* service, LPCTSTR uid, size_t uid_size, LPCTSTR passwd, size_t passwd_size) = 0;
		virtual void					close(class CAquaMessageSession* session) = 0;

		virtual CAquaMessageSession*	open_udp(int family) = 0;
		virtual void					close_udp(class CAquaMessageSession* session) = 0;
		virtual int						open_udp_bind(const char* service, int family) = 0;
		virtual void					close_udp_bind(const char* service) = 0;
		virtual int						add_user(LPCTSTR uid, size_t uid_size, LPCTSTR passwd, size_t passwd_size) = 0;
		virtual int						delete_user(LPCTSTR uid, size_t uid_size) = 0;
		virtual int						delete_alluser(void) = 0;
		virtual CAquaMessageUser		reference_user(LPCTSTR uid, size_t uid_size) const = 0;
		virtual CAquaMessageUser*		get_user(LPCTSTR uid, size_t uid_size) = 0;
		virtual const CAquaMessageUser*	get_user(LPCTSTR uid, size_t uid_size) const = 0;
		virtual int						add_ExtensionCallFunction(const tagExtensionCallFunction& ExtensionCallFunction) = 0;
		virtual int						delete_ExtensionCallFunction(int type) = 0;
		virtual int						delete_allExtensionCallFunction(void) = 0;
	};
	//==============================================
	// class CAquaMessageSystem
	//==============================================
	/**
	 * @class CAquaMessageSystem
	 * @brief bZ[WVXeNX
	 * @note bZ[WɊւȉ̋@\𓝊܂<br>
	 * E[UZbV̊Ǘ<br>
	 * E[ŮǗ<br>
	 * [UCAquaMessageSystemŊǗꂽZbVAquaNetworkSystem::CAquaNetworkSession͎gp
	 * ԐړIȃZbVNXłCAquaMessageSessionNXIuWFNggp
	 */
	class CAquaMessageSystem : public CAquaMessageSystemAbstract
	{
	private:
		eAquaMessageSystemServiceType						m_service_type;						/// T[rX
		AquaNetworkSystem::CAquaNetworkSessionManager*		m_nsm;								/// lbg[NVXeIuWFNg
		CAquaMessageUserManager								m_mum;								/// [UǗIuWFNg
		CAquaMessageUserManager								m_mum_auth;							/// Fؗp[UǗIuWFNg

		TCHAR												m_error_msg[512];					/// ŏIG[bZ[W
		std::vector<tagExtensionCallFunction>				m_ExtensionCallFunctionArray;		/// gpMf[^R[obN֐Xg


		
		/// MCxg̃R[obN֐
		static int cbfunc_Recv(AquaNetworkSystem::CAquaNetworkSession* session, short ev, void* pParam);

		/// UDPMCxg̃R[obN֐
		static int cbfunc_udp_Recv(AquaNetworkSystem::CAquaNetworkUDPSession* session, short ev, void* pParam);

		/// [UF؂
		int			auth(CAquaMessageSession* session, LPCTSTR uid, size_t uid_size, LPCTSTR passwd, size_t passwd_size);

		/// ZbVCxgʒm
		int			notify_session_event(AquaNetworkSystem::CAquaNetworkSession* session, short ev);

		/// UDPZbVCxgʒm
		int			notify_udp_session_event(AquaNetworkSystem::CAquaNetworkUDPSession* session, short ev, const struct sockaddr* sa, socklen_t salen);

		//==============================================
		// parse message method 
		//==============================================
		/// Mf[^̉͊֐
		int			parse_message(AquaNetworkSystem::CAquaNetworkSession* session, const unsigned char* buf, int num);
		/// MbZ[Wf[^̉͊֐(Fؗv)
		int			parse_message_data_user_auth_request(AquaNetworkSystem::CAquaNetworkSession* session, const void* pMessageData, size_t nMessageData);
		/// MbZ[Wf[^̉͊֐(F،)
		int			parse_message_data_user_auth_result(AquaNetworkSystem::CAquaNetworkSession* session, const void* pMessageData, size_t nMessageData);
		/// MbZ[Wf[^̉͊֐([UbZ[W)
		int			parse_message_data_user_message(AquaNetworkSystem::CAquaNetworkSession* session, const void* pMessageData, size_t nMessageData);
		/// MbZ[Wf[^̉͊֐([U)
		int			parse_message_data_user_state(AquaNetworkSystem::CAquaNetworkSession* session, const void* pMessageData, size_t nMessageData);

		/// UDPMf[^̉͊֐
		int			parse_udp_message(AquaNetworkSystem::CAquaNetworkUDPSession* session, const unsigned char* buf, int num, const struct sockaddr* sa, socklen_t salen);

	public:
		/// RXgN^
		CAquaMessageSystem();
		/// fXgN^
		virtual ~CAquaMessageSystem();

		/// ʐMpZbV̍쐬
		int						CreateSessions(int session_max);
		/// ʐM҂󂯗pZbV̍쐬
		int						CreateListenSessions(int listen_session_max);
		/// UDPʐMpZbV̍쐬
		int						CreateUDPSessions(int udp_session_max);
		/// ʐMpZbV̍폜
		void					ReleaseSessions(void);
		/// ʐM҂󂯗pZbV̍폜
		void					ReleaseListenSessions(void);
		/// UDPʐMpZbV̍폜
		void					ReleaseUDPSessions(void);

		/// SocketEvent̏
		int						InitSocketEvent(void);
		/// SocketEvent̔M
		int						DispatchSocketEvent(void);
		/// Í̏
		int						InitCrypt(void);

		//==============================================
		// server service method
		//==============================================
		/// ZbV̑҂󂯂Jn
		int						listen(const char* service, int family, eAquaMessageSystemCryptoAlgorithm CryptoAlgorithm = AQUAMESSYS_CRYPTO_ALGORITHM_NONE);
		/// ZbV̑҂󂯂I
		void					close_listen(const char* service);

		/// Fؗp[UXg[h
		int						load_auth_user_list(LPCTSTR lpFileName);

		//==============================================
		// client service method
		//==============================================
		/// ZbVւ̐ڑ
		CAquaMessageSession*	connect(const char* hostname, const char* service);
		/// ZbVւ̐ڑsA[UFؗv𑗐M܂
		CAquaMessageSession*	connect(const char* hostname, const char* service, LPCTSTR uid, size_t uid_size, LPCTSTR passwd, size_t passwd_size);
		/// ZbVւ̐ؒfs
		void					close(class CAquaMessageSession* session);

		/// UDPZbVJ
		CAquaMessageSession*	open_udp(int family);
		/// UDPZbV̐ؒfs
		void					close_udp(class CAquaMessageSession* session);
		/// UDPBindZbVJ
		int						open_udp_bind(const char* service, int family);
		/// UDPBindZbV
		void					close_udp_bind(const char* service);

		//==============================================
		// user manager method 
		//==============================================
		/// [U̒ǉ
		int						add_user(LPCTSTR uid, size_t uid_size, LPCTSTR passwd, size_t passwd_size);
		/// wuid([UID)̃[U폜
		int						delete_user(LPCTSTR uid, size_t uid_size);
		/// S[U̒ǉ
		int						delete_alluser(void);
		/// [UQƂ
		CAquaMessageUser		reference_user(LPCTSTR uid, size_t uid_size) const;
		/// [Ȕ擾
		CAquaMessageUser*		get_user(LPCTSTR uid, size_t uid_size);
		/// [Ȕ擾(const)
		const CAquaMessageUser*	get_user(LPCTSTR uid, size_t uid_size) const;
		
		//==============================================
		// ExtensionCallFunction
		//==============================================
		/// gbZ[Wf[^ǉ
		int						add_ExtensionCallFunction(const tagExtensionCallFunction& ExtensionCallFunction);
		/// w肵gbZ[Wf[^폜
		int						delete_ExtensionCallFunction(int type);
		/// SgbZ[Wf[^폜
		int						delete_allExtensionCallFunction(void);


	};
}

using namespace AquaMessageSystem;
typedef CAquaMessageSystem*			(*ty_AquaMessageSystemCreate)();

#endif // _AQUA_MESSAGE_SYSTEM_H_
